﻿Imports System.Collections.Specialized
Imports System.Security
Imports System.Text
Imports System.Web
Imports EncryptionMVC.Security.Encryption.Utility.Interfaces
Imports EncryptionMVC.Security.Encryption.Utility

Namespace Security.Encryption.QueryString
    ''' <summary>
    ''' Provides a secure means for transfering data within a query string.
    ''' </summary>
    Public Class SecureQueryString
        Inherits NameValueCollection

        Private timeStampKey As String = "__TS__"
        Private dateFormat As String = "G"
        Private mEncryptionUtil As IEncryptionUtility
        Private m_expireTime As DateTime = DateTime.MaxValue

        ''' <summary>
        ''' Creates an instance with a specified key.
        ''' </summary>
        ''' <param name="key">The key used for cryptographic functions, required 16 chars in length.</param>
        Public Sub New(ByVal key As String)
            MyBase.New()
            mEncryptionUtil = New EncryptionUtility(key)
        End Sub

        ''' <summary>
        ''' Creates an instance with a specified key and an encrypted query string.
        ''' </summary>
        ''' <param name="key">The key used for cryptographic functions, required 16 chars in length.</param>
        ''' <param name="queryString">An encrypted query string generated by a <see cref="SecureQueryString"/> instance.</param>
        Public Sub New(ByVal key As String, ByVal queryString As String)
            Me.New(key)
            Deserialize(DecryptAndVerify(queryString))
            CheckExpiration()
        End Sub

        ''' <summary>
        ''' Returns a encrypted query string.
        ''' </summary>
        ''' <returns></returns>
        Public Overloads Overrides Function ToString() As String
            Return EncryptAndSign(Serialize())
        End Function

        Private Sub Deserialize(ByVal queryString As String)
            Dim nameValuePairs As String() = queryString.Split("&"c)
            For i As Integer = 0 To nameValuePairs.Length - 1
                Dim nameValue As String() = nameValuePairs(i).Split("="c)
                If nameValue.Length = 2 Then
                    MyBase.Add(nameValue(0), nameValue(1))
                End If
            Next

            If MyBase.GetValues(timeStampKey) IsNot Nothing Then
                Dim strExpireTime As String() = MyBase.GetValues(timeStampKey)
                m_expireTime = Convert.ToDateTime(strExpireTime(0))
            End If
        End Sub

        Private Function Serialize() As String
            Dim sb As New StringBuilder()
            For Each key As String In MyBase.AllKeys
                sb.Append(key)
                sb.Append("="c)
                sb.Append(MyBase.GetValues(key)(0).ToString())
                sb.Append("&"c)
            Next

            sb.Append(timeStampKey)
            sb.Append("="c)
            sb.Append(m_expireTime.ToString(dateFormat))

            Return sb.ToString()
        End Function

        Private Function DecryptAndVerify(ByVal input As String) As String
            Return mEncryptionUtil.Decrypt(input)
        End Function

        Private Function EncryptAndSign(ByVal input As String) As String
            Return mEncryptionUtil.Encrypt(input)
        End Function

        Private Sub CheckExpiration()
            If DateTime.Compare(m_expireTime, DateTime.Now) < 0 Then
                Throw New ExpiredQueryStringException()
            End If
        End Sub

        ''' <summary>
        ''' Gets or sets the timestamp in which this string should expire
        ''' </summary>
        Public Property ExpireTime() As DateTime
            Get
                Return m_expireTime
            End Get
            Set(ByVal value As DateTime)
                m_expireTime = value
            End Set
        End Property
    End Class
End Namespace

